"""
This test the deployment of several integration test
"""

import subprocess
import re
import logging
import ssl
import socket
import os

logging.basicConfig(level=logging.DEBUG)
mylogger = logging.getLogger(__name__)

email = os.getenv("BIGIP_EMAIL", "test@test.test")
user = os.getenv("BIGIP_USERNAME")
password = os.getenv("BIGIP_PASSWORD")
bigip_list = os.getenv("BIGIP_LIST")
partition = os.getenv("BIGIP_PARTITION")
clientssl_parent = os.getenv("BIGIP_CLIENTSSL_PARENT")
vs_list = os.getenv("BIGIP_VS_LIST")
device_group = os.getenv("BIGIP_DEVICE_GROUP", "fail-sync")
iapp = os.getenv("BIGIP_IAPP")
custom_partition = os.getenv("BIGIP_CUSTOM_PARTITION", "Common")
custom_vs_list = os.getenv("BIGIP_CUSTOM_VS_LIST")


def validate_certbot_certificate_delivery(stdout: str, stderr: str):
    # from https://www.regextester.com/96683
    regex_date = r"([12]\d{3}-(0[1-9]|1[0-2])-(0[1-9]|[12]\d|3[01]))"
    regex_cert_expire_indicator = "(?<=certificate expires on )"
    regex_check_result = regex_cert_expire_indicator + regex_date
    match = re.search(regex_check_result, stdout)
    if match is not None:
        mylogger.info("New Certificate was created. Expiration Date:" + match.group(1))
        return True
    else:
        mylogger.error(
            "Certificate aquirement failed: \n stdout:\n"
            + stdout
            + "\n sterr:\n"
            + stderr
        )
        return False


def get_certbot_certificate(domain: str):
    file_path = f"/etc/letsencrypt/live/{domain}/cert.pem"

    with open(file_path, "r") as cert_file:
        return cert_file.read()


def is_delivered_cert_equal_deployed_cert(domain):
    # check if certificate has been deployed
    delivered_cert = get_certbot_certificate(domain)
    socket.setdefaulttimeout(
        5
    )  # set the timeout low to fail quickly if no certificate is served by the server

    # use sni in ssl communication
    port = 443
    conn = ssl.create_connection((domain, port))
    context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
    sock = context.wrap_socket(conn, server_hostname=domain)
    deployed_cert = ssl.DER_cert_to_PEM_cert(sock.getpeercert(True))
    mylogger.debug(f"delivered_cert: \n {delivered_cert}")
    mylogger.debug(f"deployed_cert: \n {deployed_cert}")
    return delivered_cert == deployed_cert


def test_response_evaluation():
    stdout_sample = 'IMPORTANT NOTES:\n - Congratulations! Your certificate and chain have been saved at:\n   /etc/letsencrypt/live/example1.on.at/fullchain.pem\n   Your key file has been saved at:\n   /etc/letsencrypt/live/example1.on.at/privkey.pem\n   Your certificate will expire on 2019-12-10.To obtain a new or tweaked\n   version of this certificate in the future, simply run certbot\n   again. To non-interactively renew *all* of your certificates, run\n   "certbot renew"\n'
    stderr_sample = ""
    validate_certbot_certificate_delivery(stdout_sample, stderr_sample)


def test_certbot_bigip_server_cert():
    domain = "test01.certbot.on.at"
    response = subprocess.run(
        (
            f"certbot --non-interactive --expand --email '{email}' --agree-tos"
            " -a certbot-bigip:bigip -i certbot-bigip:bigip"
            f" --certbot-bigip:bigip-list '{bigip_list}'"
            f" --certbot-bigip:bigip-username '{user}'"
            f" --certbot-bigip:bigip-password '{password}'"
            f" --certbot-bigip:bigip-partition '{partition}'"
            f" --certbot-bigip:bigip-clientssl-parent '{clientssl_parent}'"
            f" --certbot-bigip:bigip-vs-list '{vs_list}'"
            f" --certbot-bigip:bigip-device-group '{device_group}'"
            f" --certbot-bigip:bigip-iapp '{iapp}'"
            f" -d {domain}"
            " -v --staging --debug --force-renewal"
        ),
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        shell=True,
    )

    mylogger.debug(response.stdout.decode("utf-8"))

    # check if new certificate was created
    if not validate_certbot_certificate_delivery(
        response.stdout.decode("utf-8"), response.stderr.decode("utf-8")
    ):
        assert False
    # check if process has finished correctly
    response.check_returncode()

    # check if certificate has been deployed
    assert is_delivered_cert_equal_deployed_cert(
        domain
    ), "did you assign the ssl profile to the virtualserver? this step is not automated."


def test_certbot_bigip_deployment_in_custompartition_and_folders():
    domain = "test02.certbot.on.at"
    response = subprocess.run(
        (
            f"certbot --non-interactive --expand --email '{email}' --agree-tos"
            " -a certbot-bigip:bigip -i certbot-bigip:bigip"
            f" --certbot-bigip:bigip-list '{bigip_list}'"
            f" --certbot-bigip:bigip-username '{user}'"
            f" --certbot-bigip:bigip-password '{password}'"
            f" --certbot-bigip:bigip-partition '{custom_partition}'"
            f" --certbot-bigip:bigip-clientssl-parent '{clientssl_parent}'"
            f" --certbot-bigip:bigip-vs-list '{custom_vs_list}'"
            f" --certbot-bigip:bigip-device-group '{device_group}'"
            f" -d {domain}"
            " -v --staging --debug --force-renewal"
        ),
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        shell=True,
    )

    # check if new certificate was created
    if not validate_certbot_certificate_delivery(
        response.stdout.decode("utf-8"), response.stderr.decode("utf-8")
    ):
        assert False

    # check if process has finished correctly
    response.check_returncode()

    # check if certificate has been deployed
    assert is_delivered_cert_equal_deployed_cert(
        domain
    ), "did you assign the ssl profile to the virtualserver? this step is not automated."


def test_wildcard_deployment_with_bluecat_validation():
    domain = "*.certbot.on.at"
    response = subprocess.run(
        (
            "certbot"
            " --non-interactive"
            f" -d {domain}"
            " --staging"
            " --expand"
            f" --email '{email}'"
            " --agree-tos"
            " --no-eff-email"
            " -v"
            " --debug"
            " --force-renewal"
            # authentication
            " -a certbot-bluecat:bluecat"
            " -i certbot-bigip:bigip"
            f" --certbot-bigip:bigip-list '{bigip_list}'"
            f" --certbot-bigip:bigip-username '{user}'"
            f" --certbot-bigip:bigip-password '{password}'"
            f" --certbot-bigip:bigip-partition '{partition}'"
            f" --certbot-bigip:bigip-clientssl-parent '{clientssl_parent}'"
            f" --certbot-bigip:bigip-vs-list '{vs_list}'"
            f" --certbot-bigip:bigip-device-group '{device_group}'"
            f" --certbot-bigip:bigip-iapp '{iapp}'"
        ),
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        shell=True,
    )

    # check if new certificate was created
    if not validate_certbot_certificate_delivery(
        response.stdout.decode("utf-8"), response.stderr.decode("utf-8")
    ):
        assert False

    # check if process has finished correctly
    response.check_returncode()

    # check if certificate has been deployed
    assert is_delivered_cert_equal_deployed_cert(
        domain.strip("*.")
    ), "did you assign the ssl profile to the virtualserver? this step is not automated."



import logging
import os

from certbot_bigip.configurator import BigipConfigurator

logging.basicConfig(level=logging.DEBUG)
mylogger = logging.getLogger(__name__)

email = os.getenv("BIGIP_EMAIL", "test@test.test")
user = os.getenv("BIGIP_USERNAME")
password = os.getenv("BIGIP_PASSWORD")
bigip_list = os.getenv("BIGIP_LIST")
partition = os.getenv("BIGIP_PARTITION", "Intern")
clientssl_parent = os.getenv("BIGIP_CLIENTSSL_PARENT")
vs_list = os.getenv("BIGIP_VS_LIST")
device_group = os.getenv("BIGIP_DEVICE_GROUP", "fail-sync")
iapp = os.getenv("BIGIP_IAPP")
custom_partition = os.getenv("BIGIP_CUSTOM_PARTITION", "Common")
custom_vs_list = os.getenv("BIGIP_CUSTOM_VS_LIST")

from OpenSSL import crypto, SSL


def cert_gen(
    emailAddress="emailAddress",
    commonName="commonName",
    countryName="NT",
    localityName="localityName",
    stateOrProvinceName="stateOrProvinceName",
    organizationName="organizationName",
    organizationUnitName="organizationUnitName",
    serialNumber=0,
    validityStartInSeconds=0,
    validityEndInSeconds=10 * 365 * 24 * 60 * 60,
    KEY_FILE="private.key",
    CERT_FILE="selfsigned.crt",
):
    # can look at generated file using openssl:
    # openssl x509 -inform pem -in selfsigned.crt -noout -text
    # create a key pair
    k = crypto.PKey()
    k.generate_key(crypto.TYPE_RSA, 4096)
    # create a self-signed cert
    cert = crypto.X509()
    cert.get_subject().C = countryName
    cert.get_subject().ST = stateOrProvinceName
    cert.get_subject().L = localityName
    cert.get_subject().O = organizationName
    cert.get_subject().OU = organizationUnitName
    cert.get_subject().CN = commonName
    cert.get_subject().emailAddress = emailAddress
    cert.set_serial_number(serialNumber)
    cert.gmtime_adj_notBefore(0)
    cert.gmtime_adj_notAfter(validityEndInSeconds)
    cert.set_issuer(cert.get_subject())
    cert.set_pubkey(k)
    cert.sign(k, "sha512")
    with open(CERT_FILE, "wt") as f:
        f.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert).decode("utf-8"))
    with open(KEY_FILE, "wt") as f:
        f.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, k).decode("utf-8"))


#########################################################
# Instanciating an object without using the certbot-CLI #
#########################################################


class Config(object):
    def __init__(self):
        self.certbot_bigip_list = bigip_list
        self.certbot_bigip_username = user
        self.certbot_bigip_password = password
        self.certbot_bigip_partition = partition
        self.certbot_bigip_iapp = iapp
        self.certbot_bigip_clientssl_parent = clientssl_parent
        self.certbot_bigip_vs_list = vs_list
        self.certbot_bigip_device_group = device_group
        self.certbot_bigip_apm = False
        self.certbot_bigip_verify_ssl = False
        self.backup_dir = "/tests/backup"
        self.strict_permissions = False
        self.temp_checkpoint_dir = "/tests/backup"
        self.in_progress_dir = "/tests/backup"


def test_upload_cert_to_bigip():
    config = Config()

    configurator = BigipConfigurator(config, "certbot_bigip")
    configurator.prepare()
    configurator.cert_chain_name = "test_chain"

    domain = "test01.certbot.on.at"

    from pathlib import Path

    Path("tests/test_certificates").mkdir(parents=True, exist_ok=True)

    cert_path = "tests/test_certificates/test_certbot_ong_at.pem"
    key_path = "tests/test_certificates/test_certbot_ong_at_key.pem"
    chain_path = "tests/test_certificates/test_chain.pem"

    cert_gen(CERT_FILE=cert_path, KEY_FILE=key_path)

    if configurator.deploy_cert(
        domain,
        cert_path,
        key_path,
        cert_path,
    ):
        assert True



import os

from setuptools import find_packages
from setuptools import setup


if os.environ.get("GITHUB_RELEASE_TAG"):
    version = os.environ["GITHUB_RELEASE_TAG"]
elif os.environ.get("CI_JOB_ID"):
    version = os.environ["CI_JOB_ID"]
else:
    try:
        with open("PKG-INFO") as f:
            for line in f:
                if line.startswith("Version: "):
                    version = line.split("Version: ")[1].split("\n")[0]
                    break
    except IOError:
        version = "unkown version"

with open("README.md", "r") as fh:
    long_description = fh.read()

install_requires = [
    "acme",
    "certbot",
    "f5-icontrol-rest",
    "f5-sdk",
    "requests",
    "setuptools>=1.0",
    "zope.component",
    "zope.interface",
]

setup(
    name="certbot-bigip",
    version=version,
    description="F5 BIG-IP plugin for Certbot",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Certbot Team @ Open Networks",
    author_email="certbot@on.at",
    license="Apache License 2.0",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Plugins",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Security",
        "Topic :: System :: Installation/Setup",
        "Topic :: System :: Networking",
        "Topic :: System :: Systems Administration",
        "Topic :: Utilities",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    extras_require={},
    entry_points={
        "certbot.plugins": [
            "bigip = certbot_bigip.configurator:BigipConfigurator",
        ],
    },
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
    test_suite="certbot_bigip",
)
